﻿using FCS.Common;
using FCS.Interface;
using FCS.Models;
using FCS.Models.DTO;
using FCS.Models.Entity;
using HtmlAgilityPack;
using Quartz;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace FCS.Crawler.Basketball
{
    /// <summary>
    /// 篮球联盟排名服务
    /// </summary>
    public class B_LeagueRankingJob : CommonJob, IJob
    {
        private static List<B_LeagueRanking> LeagueRankingList = new List<B_LeagueRanking>();
        private static List<DataItemDetail> DateItem = new List<DataItemDetail>();
        private static List<B_Team> TeamList = new List<B_Team>();
        B_LeagueRanking g;
        public B_LeagueRankingJob()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            g = new B_LeagueRanking();
        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
        }


        #region 更新数据
        /// <summary>
        /// 获取所有数据
        /// </summary>
        public void GetAll()
        {
            GetSqlString(GetAllEventUrl, "");
        }

        /// <summary>
        /// 更新数据
        /// </summary>
        /// <param name="Year"></param>
        public void Click(string Year = "")
        {
            if (Year == "")
            {
                Year = DateTime.Now.Year.ToString().Substring(2, 2);
            }
            var sql = string.Format(GetNewEventUrl, Year);
            GetSqlString(sql, Year);
        }
        #endregion

        /// <summary>
        /// 解析数据
        /// </summary>
        /// <param name="sqlstring"></param>
        private void GetSqlString(string sqlstring, string Year)
        {
            TeamList = services.Query<B_Team>().ToList();
            var ds = services.Query<B_Grouping>("and Remark is not null", "", "distinct EventId,Season,Remark").ToList();
            if (Year != "")
            {
                ds = services.Query<B_Grouping>("and Remark is not null  and Season like '%" + Year + "%'", "", "distinct EventId,Season,Remark").ToList();
            }

            int max = ds.Count;
            int num = 0;
            ds.ForEach(async p =>
            {
                await Task.Run(() =>
                 {
                     var url = p.Remark.ToString().Replace("saiAll.do", "pai.do");
                     var eventId = p.EventId;
                     var season = p.Season;
                     GetData(url, eventId, season);

                 });
                lock (g)
                {
                    num++;
                    Monitor.Pulse(g); //完成，通知等待队列,告知已完，执行下一个。
                }

            });

            lock (g)
            {
                while (num < max)
                {
                    Monitor.Wait(g);//等待
                }
            }
            Trace.WriteLine("获取B_LeagueRanking 完结");
            try
            {
                if (Year != "")
                {
                    services.Delete<B_LeagueRanking>(" and Season like '%" + Year + "%'");
                }

                services.SqlBulkCopyAdd<B_LeagueRanking>(LeagueRankingList);
            }
            catch (Exception ex)
            {

                throw;
            }

        }
        /// <summary>
        /// 解析排行榜数据
        /// </summary>
        /// <param name="url"></param>
        /// <param name="eventId"></param>
        /// <param name="season"></param>
        private void GetData(string url, string eventId, string season)
        {
            HtmlParameterDTO dtomodel = new HtmlParameterDTO();
            dtomodel.Url = url;
            HtmlDocument doc = CommonHelper.GetHtmlHtmlDocument(dtomodel);
            if (doc.DocumentNode.InnerHtml == "Termination" || doc.DocumentNode.InnerHtml == "" || doc.DocumentNode.SelectNodes("//*[@class='paiming']") == null)
            {
                return;
            }
            var team = doc.DocumentNode.SelectNodes("//*[@class='paiming']");
            foreach (var item in team)
            {
                HtmlDocument itemdoc = new HtmlDocument();
                itemdoc.LoadHtml(item.InnerHtml);
                var trList = itemdoc.DocumentNode.SelectNodes("//table/tr").ToList();
                var TypeName = "";
                if (itemdoc.DocumentNode.InnerText.Contains("东部"))
                {
                    TypeName = "东部排行";
                }
                else
                {
                    TypeName = "西部排行";
                }
                trList.ForEach(p =>
                {
                    if (!p.InnerText.Contains("<th"))
                    {
                        HtmlDocument trdoc = new HtmlDocument();
                        trdoc.LoadHtml(p.InnerHtml);
                        if (doc.DocumentNode.SelectNodes("//td") == null)
                            return;
                        var tdList = trdoc.DocumentNode.InnerHtml.Contains("td") ? trdoc.DocumentNode.SelectNodes("//td").ToList() : null;
                        if (tdList != null && tdList.Count() > 1)
                        {
                            B_LeagueRanking ranking = new B_LeagueRanking();
                            ranking.Id = Guid.NewGuid().ToString();
                            ranking.Sort = int.Parse(tdList[0].InnerText.ToString());
                            ranking.TeamId = TeamList.Where(o => o.Name == tdList[1].InnerText.Trim().ToString()).Count() > 0 ? TeamList.Where(o => o.Name == tdList[1].InnerText.Trim().ToString()).FirstOrDefault().Id : "";
                            ranking.EventId = eventId;
                            ranking.Season = season;
                            ranking.Win = int.Parse(tdList[2].InnerText.Trim().ToString());
                            ranking.Lose = int.Parse(tdList[3].InnerText.Trim().ToString());
                            ranking.WinRate = tdList[4].InnerText.Trim().ToString();
                            ranking.WinDivide = tdList[5].InnerText.Trim().ToString();
                            ranking.ScoreAvg = tdList[6].InnerText.Trim().ToString();
                            ranking.LossAvg = tdList[7].InnerText.Trim().ToString();
                            ranking.League = tdList[8].InnerText.Trim().ToString();
                            ranking.Division = tdList[9].InnerText.Trim().ToString();
                            ranking.Home = tdList[10].InnerText.Trim().ToString();
                            ranking.Visiting = tdList[11].InnerText.Trim().ToString();
                            ranking.NearlyTen = tdList[12].InnerText.Trim().ToString();
                            ranking.Status = tdList[13].InnerText.Trim().ToString();
                            ranking.Remark = TypeName;
                            LeagueRankingList.Add(ranking);
                        }

                    }
                });
            }
        }
        #region SQL语句
        /// <summary>
        /// 获取所有的赛事
        /// </summary>
        private static string GetAllEventUrl = @"select distinct EventId,Season,Remark  from B_Grouping where Remark is not null";
        /// <summary>
        /// 获取最新两年的数据赛事
        /// </summary>
        private static string GetNewEventUrl = @"select distinct EventId,Season,Remark  from B_Grouping where Remark is not null and Season like '%{0}%'";

        #endregion
    }
}
