﻿using System;
using System.Collections.Generic;
using FCS.Common;
using FCS.Interface;
using FCS.Models;
using HtmlAgilityPack;
using Quartz;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Threading;
using System.Diagnostics;

namespace FCS.Crawler.ZCMatchRankingList
{
    /// <summary>
    /// 进球数单双统计
    /// </summary>
    public class GoalSingleDbListJob : CommonJob, IJob
    {

        private static List<F_SingleDoubleList> SingleDoubleLists = new List<F_SingleDoubleList>();
        private static List<F_Team> TeamList = new List<F_Team>();
        F_SingleDoubleList g;

        public GoalSingleDbListJob()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            g = new F_SingleDoubleList();
        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            GetAll();
        }

        #region  更新数据
        /// <summary>
        /// 获取所有数据
        /// </summary>
        public void GetAll()
        {
            GetSqlString(GetAllEventUrl, "");
        }

        /// <summary>
        /// 更新数据，默认为今年数据
        /// </summary>
        /// <param name="Year"></param>
        public void Click(string Year = "")
        {
            if (Year == "")
            {
                Year = DateTime.Now.Year.ToString();
            }
            var sql = string.Format(GetNewEventUrl, Year);
            GetSqlString(sql, Year);
        }
        #endregion


        /// <summary>
        /// 解析数据
        /// </summary>
        /// <param name="sqlstring"></param>
        private void GetSqlString(string sqlstring, string Year)
        {
            TeamList = services.GetTeamList();
            var searchsql = "and Remark is not null  ";
            if (Year != "")
            {
                searchsql += "and Season like '%" + Year + "%'";
            }
            var ds = services.Query<F_Grouping>(searchsql, "", "distinct EventId,Season,Remark").ToList();
            if (ds != null && ds.Count > 0)
            {
                int max = ds.Count;
                int num = 0;
                ds.ForEach(async p =>
                {
                    //拿去赛事组的链接
                    var url = p.Remark.ToString();
                    var eventId = p.EventId.ToString();
                    var season = p.Season.ToString();
                    ////获取到总进球单双的链接
                    await Task.Run(() =>
                    {
                        HtmlDocument doc = CommonHelper.GetHtml(url, new Dictionary<string, string>(), "", "", 10000, 100);
                        var TJList = doc.DocumentNode.SelectNodes("//*[@class='tongji_list']");
                        if (TJList != null && TJList.Count() != 0 && TJList[0].InnerText.Contains("赛事统计"))
                        {
                            HtmlDocument doc1 = new HtmlDocument();
                            doc1.LoadHtml(TJList[0].InnerHtml);
                            var liList = doc1.DocumentNode.SelectNodes("//ul/li/a").ToList();
                            var Glod = liList.Where(o => o.InnerText.Contains("进球单双")).First();
                            if (Glod != null)
                            {
                                var jqdsurl = Glod.Attributes.SingleOrDefault(a => a.Name.Equals("href")).Value.ToString();
                                Task.Run(() =>
                                {
                                    GetJQDSHtml(jqdsurl, eventId, season);
                                });
                            }
                        }
                    });
                    lock (g)
                    {
                        num++;
                        Monitor.Pulse(g); //完成，通知等待队列,告知已完，执行下一个。
                    }
                });
                lock (g)
                {
                    while (num < max)
                    {
                        Monitor.Wait(g);//等待
                    }
                }
                Trace.WriteLine("获取GoalSingleDbListJob 完结");
                try
                {
                    if (Year != "")
                    {
                        services.Delete<F_SingleDoubleList>(" and Season like '%" + Year + "%'");
                    }
                    services.SqlBulkCopyAdd<F_SingleDoubleList>(SingleDoubleLists);

                    Trace.WriteLine("F_SingleDoubleList更新完毕");
                }
                catch (Exception ex)
                {
                    throw;
                }
            }
        }


        /// <summary>
        /// 解析进球单双页面
        /// </summary>
        /// <param name="url"></param>
        /// <param name="eventId"></param>
        /// <param name="season"></param>
        public void GetJQDSHtml(string url, string eventId, string season)
        {
            if (url.Contains("wwaattssuunn"))
            {
                return;
            }
            HtmlDocument doc = CommonHelper.GetHtml(url, new Dictionary<string, string>(), "", "", 1000, 100);

            var trData = doc.DocumentNode.SelectNodes("//*[@class='table_out']/table/tbody/tr");
            if (trData == null || trData.Count() == 0) return;
            //读取进球单双信息并加入到数据库
            foreach (var tritem in trData)
            {
                try
                {
                    HtmlDocument doc11 = new HtmlDocument();
                    doc11.LoadHtml(tritem.InnerHtml);
                    var tdList = doc11.DocumentNode.SelectNodes("//td");
                    if (tdList == null || tdList.Count == 0) continue;
                    F_SingleDoubleList f_SingleDoubleList = new F_SingleDoubleList();
                    f_SingleDoubleList.Id = Guid.NewGuid().ToString();
                    f_SingleDoubleList.EventId = eventId;
                    f_SingleDoubleList.Season = season;
                    f_SingleDoubleList.Sort = tdList[0].InnerText.Trim() == null ? 0 : int.Parse(tdList[0].InnerText.Trim().ToString());
                    var TeamName = tdList[1].InnerText.Trim().ToString();
                    var Team = TeamList.Where(o => o.Name == TeamName);
                    f_SingleDoubleList.TeamId = Team.Count() > 0 ? Team.FirstOrDefault().Id : "";
                    if (Team.Count() == 0)
                    {
                        f_SingleDoubleList.Remark = TeamName;
                    }
                    f_SingleDoubleList.OneNum = tdList.Count() > 3 ? int.Parse(tdList[2].InnerText.Trim().ToString()) : 0;
                    f_SingleDoubleList.ZeroNum = tdList.Count() > 4 ? int.Parse(tdList[3].InnerText.Trim().ToString()) : 0;
                    f_SingleDoubleList.TwoNum = tdList.Count() > 5 ? int.Parse(tdList[4].InnerText.Trim().ToString()) : 0;
                    f_SingleDoubleList.ThreeNum = tdList.Count() > 6 ? int.Parse(tdList[5].InnerText.Trim().ToString()) : 0;
                    f_SingleDoubleList.FourNum = tdList.Count() > 7 ? int.Parse(tdList[6].InnerText.Trim().ToString()) : 0;
                    f_SingleDoubleList.OddNum = tdList.Count() > 8 ? int.Parse(tdList[7].InnerText.Trim().ToString()) : 0;
                    f_SingleDoubleList.EvenNum = tdList.Count() > 9 ? int.Parse(tdList[8].InnerText.Trim().ToString()) : 0;
                    SingleDoubleLists.Add(f_SingleDoubleList);
                }
                catch (Exception ex)
                {
                    throw;
                }
            }

        }




        #region SQL语句

        /// <summary>
        /// 获取所有的赛事
        /// </summary>
        private static string GetAllEventUrl = @"select distinct EventId,Season,Remark  from F_Grouping where Remark is not null";

        /// <summary>
        /// 获取最新两年的数据赛事
        /// </summary>
        private static string GetNewEventUrl = @"select distinct EventId,Season,Remark  from F_Grouping where Remark is not null and Season like '%{0}%'";


        private static string DeleteNewEventUrl = @"delete F_SingleDoubleList where  Season like '{0}%'";
        #endregion

    }
}
