﻿using System;
using System.Collections.Generic;
using FCS.Common;
using FCS.Interface;
using FCS.Models;
using HtmlAgilityPack;
using Quartz;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Threading;
using System.Diagnostics;

namespace FCS.Crawler.ZCMatchRankingList
{
    /// <summary>
    /// 射手榜排行
    /// </summary>
    public class ShooterListJob : CommonJob, IJob
    {

        private static List<F_ShooterList> ShooterLists = new List<F_ShooterList>();
        private static List<F_Team> TeamList = new List<F_Team>();
        private static List<F_Players> PlayerList = new List<F_Players>();
        private static List<F_Area> ContryList = new List<F_Area>();
        private static List<DataItemDetail> DateItem = new List<DataItemDetail>();
        F_ShooterList g;
        public ShooterListJob()
        {
            log = new LogHelper();
            services = IOC.Resolve<IDTOpenCode>();
            g = new F_ShooterList();
        }

        public void Execute(IJobExecutionContext context)
        {
            Config = CommonHelper.GetConfigFromDataMap(context.JobDetail.JobDataMap);
            GetAll();
        }

        #region 更新数据
        /// <summary>
        /// 获取所有数据
        /// </summary>
        public void GetAll()
        {
            GetSqlString( "");
        }

        /// <summary>
        /// 更新数据
        /// </summary>
        /// <param name="Year"></param>
        public void Click(string Year = "")
        {
            if (Year == "")
            {
                Year = DateTime.Now.Year.ToString();
            }
            GetSqlString( Year);
        }
        #endregion


        /// <summary>
        /// 解析数据
        /// </summary>
        /// <param name="sqlstring"></param>
        private void GetSqlString(string Year)
        {
            TeamList = services.GetTeamList();
            PlayerList = services.GetPlayerList();
            ContryList = services.GetContryList();
            var searchsql = "and Remark is not null  ";
            if (Year != "")
            {
                searchsql += "and Season like '%" + Year + "%'";
            }
            var ds = services.Query<F_Grouping>(searchsql, "", "distinct EventId,Season,Remark").ToList();
            if (ds != null && ds.Count > 0)
            {
                int max = ds.Count;
                int num = 0;
                ds.ForEach(async p =>
                {
                    //拿去赛事组的链接
                    var url = p.Remark.ToString();
                    var eventId = p.EventId.ToString();
                    var season = p.Season.ToString();
                    ////获取到总进球单双的链接
                    await Task.Run(() =>
                    {
                        HtmlDocument doc = CommonHelper.GetHtml(url, new Dictionary<string, string>(), "", "", 10000, 100);
                        var TJList = doc.DocumentNode.SelectNodes("//*[@class='tongji_list']");
                        if (TJList != null && TJList.Count() != 0 && TJList[0].InnerText.Contains("赛事统计"))
                        {
                            HtmlDocument doc1 = new HtmlDocument();
                            doc1.LoadHtml(TJList[0].InnerHtml);
                            var liList = doc1.DocumentNode.SelectNodes("//ul/li/a").ToList();
                            var Glod = liList.Where(o => o.InnerText.Contains("射手榜")).First();
                            if (Glod != null)
                            {
                                var shooturl = Glod.Attributes.SingleOrDefault(a => a.Name.Equals("href")).Value.ToString();
                                Task.Run(() =>
                                {
                                    GetList(shooturl, eventId, season);
                                });
                            }
                        }
                    });
                    lock (g)
                    {
                        num++;
                        Monitor.Pulse(g); //完成，通知等待队列,告知已完，执行下一个。
                    }
                });
                lock (g)
                {
                    while (num < max)
                    {
                        Monitor.Wait(g);//等待
                    }
                }
                Trace.WriteLine("获取ShooterListJob 完结");
                try
                {
                    if (Year != "")
                    {
                        services.Delete<F_ShooterList>(" and Season like '%" + Year + "%'");
                    }
                    services.SqlBulkCopyAdd<F_ShooterList>(ShooterLists);
                    Trace.WriteLine("F_ShooterList更新完毕");
                }
                catch (Exception ex)
                {
                    throw;

                }
            }
        }

        /// <summary>
        /// 解析射手榜信息
        /// </summary>
        /// <param name="url"></param>
        /// <param name="eventId"></param>
        /// <param name="season"></param>
        public void GetList(string url, string eventId, string season)
        {
            if (url.Contains("wwaattssuunn"))
            {
                return;
            }
            HtmlDocument doc = CommonHelper.GetHtml(url, new Dictionary<string, string>(), "", "", 10000, 100);

            var TongJiListData = doc.DocumentNode.SelectNodes("//*[@class='zstab']");
            if (TongJiListData == null || TongJiListData.Count() == 0) return;
            DateItem = services.GetDataItem(DataItemDetailEnum.ShooterType);
            //总榜
            var RankingAll = TongJiListData[0];
            ShooterList(RankingAll, eventId, season, ShooterEnum.AllShooter);
            //主场
            var RankingMain = TongJiListData[1];
            ShooterList(RankingMain, eventId, season, ShooterEnum.MainShooter);
            //客场
            if (TongJiListData.Count > 2)
            {
                var RankingPassenger = TongJiListData[2];
                ShooterList(RankingPassenger, eventId, season, ShooterEnum.PassengerShooter);
            }
        }

        public void ShooterList(HtmlNode ShooterAll, string eventId, string season, ShooterEnum shooterEnum)
        {
            HtmlDocument doc1 = new HtmlDocument();
            doc1.LoadHtml(ShooterAll.InnerHtml);
            var allshooterList = doc1.DocumentNode.SelectNodes("//tbody/tr");
            if (allshooterList == null || allshooterList.Count == 0) return;
            //读取射手榜信息并加入到数据库
            foreach (var tritem in allshooterList)
            {
                try
                {
                    HtmlDocument doc11 = new HtmlDocument();
                    doc11.LoadHtml(tritem.InnerHtml);
                    var tdList = doc11.DocumentNode.SelectNodes("//td");
                    if (tdList == null || tdList.Count == 0) continue;
                    F_ShooterList f_ShooterList = new F_ShooterList();
                    f_ShooterList.Id = Guid.NewGuid().ToString();
                    f_ShooterList.EventId = eventId;
                    f_ShooterList.Season = season;
                    f_ShooterList.ShooterType = DateItem.Where(o => o.ItemValue == ((int)shooterEnum).ToString()).Count() > 0 ? DateItem.Where(o => o.ItemValue == ((int)shooterEnum).ToString()).FirstOrDefault().Id : "";
                    f_ShooterList.Sort = tdList[0].InnerText.Trim() == null || tdList[0].InnerText.Trim() == "" ? 0 : int.Parse(tdList[0].InnerText.Trim().ToString());
                    var TeamName = tdList[1].InnerText.Trim().ToString();
                    var Team = TeamList.Where(o => o.Name == TeamName);
                    f_ShooterList.TeamId = Team.Count() > 0 ? Team.FirstOrDefault().Id : "";
                    if (Team.Count() == 0)
                    {
                        f_ShooterList.Remark = TeamName;
                    }
                    f_ShooterList.PlayerName = tdList.Count() > 3 ? tdList[2].InnerText.Trim().ToString() : "";
                    var Player = PlayerList.Where(o => f_ShooterList.PlayerName != "" && o.ChineseName.Contains(f_ShooterList.PlayerName));
                    f_ShooterList.PlayerId = Player.Count() > 0 ? Player.FirstOrDefault().Id : "";
                    f_ShooterList.ContryName = tdList.Count() > 4 ? tdList[3].InnerText.Trim().ToString() : "";
                    var Contry = ContryList.Where(o => f_ShooterList.ContryName != "" && o.Name == f_ShooterList.ContryName);
                    f_ShooterList.ContryId = Contry.Count() > 0 ? Contry.FirstOrDefault().Id : "";
                    f_ShooterList.Sum = tdList.Count() > 5 ? tdList[4].InnerText.Trim() == null || tdList[4].InnerText.Trim() == "" ? 0 : int.Parse(tdList[4].InnerText.Trim().ToString()) : 0;
                    f_ShooterList.PenaltyKick = tdList.Count() > 6 ? tdList[5].InnerText.Trim() == null || tdList[5].InnerText.Trim() == "" ? 0 : int.Parse(tdList[5].InnerText.Trim().ToString()) : 0;
                    f_ShooterList.HomeGoal = tdList.Count() > 7 ? tdList[6].InnerText.Trim() == null || tdList[6].InnerText.Trim() == "" ? 0 : int.Parse(tdList[6].InnerText.Trim().ToString()) : 0;
                    f_ShooterList.PassengerGoal = tdList.Count() > 8 ? tdList[7].InnerText.Trim() == null || tdList[7].InnerText.Trim() == "" ? 0 : int.Parse(tdList[7].InnerText.Trim().ToString()) : 0;
                    ShooterLists.Add(f_ShooterList);
                }
                catch (Exception ex)
                {

                    throw;
                }
            }
        }
        
    }
}
